import pytest
from pytest_mock import mocker
from mock import MagicMock, call, patch
from mykonos.keywords.element import *
from mykonos.keywords.wait import Wait
from mykonos.locator.locator_element import LocatorElement
from mykonos.keywords.management_device import ManagementDevice


def test_long_click_element():
    el = Click()
    el.long_click_element = MagicMock()
    el.long_click_element.return_value = True
    result = el.long_click_element(text=MagicMock())
    assert result == True

def test_page_should_contain_element():
    el = ExpectedConditions()
    el.page_should_contain_element = MagicMock()
    el.page_should_contain_element.return_value = True
    result = el.page_should_contain_element(className=MagicMock())
    assert result == True

def test_wait_until_page_does_not_contain():
    el = Wait()
    el.wait_until_page_does_not_contain = MagicMock()
    el.wait_until_page_does_not_contain.return_value = False
    result = el.wait_until_page_does_not_contain(className=MagicMock())
    assert result == False

def test_wait_until_page_does_not_contain_element():
    el = Wait()
    el.wait_until_page_does_not_contain_element = MagicMock()
    el.wait_until_page_does_not_contain_element.return_value = False
    result = el.wait_until_page_does_not_contain_element(className=MagicMock())
    assert result == False

def test_check_element_exists():
    el = ExpectedConditions()
    el.check_element_exists = MagicMock()
    el.check_element_exists.return_value = True
    result = el.check_element_exists(className=MagicMock())
    assert result == True

def test_check_element_not_exists():
    el = ExpectedConditions()
    el.check_element_not_exists = MagicMock()
    el.check_element_not_exists.return_value = True
    result = el.check_element_not_exists(className=MagicMock())
    assert result == True

def test_wait_until_page_contains():
    el = Wait()
    el.wait_until_page_contains = MagicMock()
    el.wait_until_page_contains.return_value = True
    result = el.wait_until_page_contains(className=MagicMock())
    assert result == True

def test_wait_until_page_contains_element():
    el = Wait()
    el.wait_until_page_contains_element = MagicMock()
    el.wait_until_page_contains_element.return_value = True
    result = el.wait_until_page_contains_element(className=MagicMock())
    assert result == True

def test_text_should_be_visible():
    el = GlobalElement()
    el.text_should_be_visible = MagicMock()
    el.text_should_be_visible.return_value = True
    result = el.text_should_be_visible(text=MagicMock())
    assert result == True

def test_wait_until_element_is_visible():
    el = Wait()
    el.wait_until_element_is_visible = MagicMock()
    el.wait_until_element_is_visible.return_value = True
    result = el.wait_until_element_is_visible(className=MagicMock())
    assert result == True


def test_page_should_not_contain_element():
    el = ExpectedConditions()
    el.page_should_not_contain_element = MagicMock()
    el.page_should_not_contain_element.return_value = True
    result = el.page_should_not_contain_element(className=MagicMock())
    assert result == True


def test_page_should_not_contain_text():
    el = ExpectedConditions()
    el.page_should_not_contain_text = MagicMock()
    el.page_should_not_contain_text.return_value = True
    result = el.page_should_not_contain_text(className=MagicMock())
    assert result == True


def test_page_should_contain_text():
    el = ExpectedConditions()
    el.page_should_contain_text = MagicMock()
    el.page_should_contain_text.return_value = True
    result = el.page_should_contain_text(text=MagicMock())
    assert result == True


def test_click_element():
    el = Click()
    el.click_element = MagicMock()
    el.click_element.return_value = True
    result = el.click_element(text=MagicMock())
    assert result == True


def test_click_element_parent_without_device():
    el = Click()
    el.click_element = MagicMock()
    el.click_element.return_value = True
    result = el.click_element(locator=MagicMock())
    assert result == True


def test_click_element_parent_with_device():
    el = Click()
    el.click_element = MagicMock()
    el.click_element.return_value = True
    result = el.click_element(device= MagicMock(),locator= MagicMock())
    assert result == True


def test_click_element_device_and_element():
    el = Click()
    d1 = MagicMock()
    text = MagicMock()
    el.click_element = MagicMock()
    el.click_element.return_value = True
    result = el.click_element(device=d1, text="Messaging")
    assert result == True


def test_input_text_with_device_parent():
    el = GlobalElement()
    el.input_text = MagicMock()
    el.input_text.return_value = True
    result = el.input_text(className=MagicMock(), input='Test')
    assert result == True


def test_input_text_with_parameter():
    el = GlobalElement()
    el.input_text = MagicMock()
    el.input_text.return_value = True
    result = el.input_text(device=MagicMock(), input='Test',className=MagicMock())
    assert result == True


def test_input_text_with_locator():
    el = GlobalElement()
    el.input_text = MagicMock()
    el.input_text.return_value = True
    result = el.input_text(className=MagicMock(), input='Test')
    assert result == True


def test_clear_text():
    el = GlobalElement()
    el.clear_text = MagicMock()
    el.clear_text.return_value = True
    result = el.clear_text(className=MagicMock())
    assert result == True


def test_clear_text_with_device():
    el = GlobalElement()
    el.clear_text = MagicMock()
    el.clear_text.return_value = True
    result = el.clear_text(device=MagicMock(), className=MagicMock())
    assert result == True


def test_clear_text_with_device_and_parent():
    el = GlobalElement()
    el.clear_text = MagicMock()
    el.clear_text.return_value = True
    result = el.clear_text(device=MagicMock(), locator=MagicMock())
    assert result == True


def test_get_text():
    el = GetConditions()
    el.get_text = MagicMock()
    el.get_text.return_value = 'return text'
    result = el.get_text(className=MagicMock())
    assert len(result) != 0

def test_get_text_with_device():
    el = GetConditions()
    el.get_text = MagicMock()
    el.get_text.return_value = 'return text'
    result = el.get_text(device=MagicMock(), className=MagicMock())
    assert len(result) != 0


def test_get_text_with_parents():
    el = GetConditions()
    el.get_text = MagicMock()
    el.get_text.return_value = 'return text'
    result = el.get_text(locator=MagicMock())
    assert len(result) != 0


def test_get_text_with_parents_device():
    el = GetConditions()
    el.get_text = MagicMock()
    el.get_text.return_value = 'return text'
    result = el.get_text(device=MagicMock(), locator=MagicMock())
    assert len(result) != 0


def test_get_attribute():
    el = GetConditions()
    el.get_element_attribute = MagicMock()
    el.get_element_attribute.return_value = 'sample return attribute'
    result = el.get_element_attribute(element=MagicMock(), className=MagicMock())
    assert len(result) != 0


def test_get_attribute_with_device():
    el = GetConditions()
    el.get_element_attribute = MagicMock()
    el.get_element_attribute.return_value = 'sample return attribute'
    result = el.get_element_attribute(device=MagicMock(), element=MagicMock(), className=MagicMock())
    assert len(result) != 0


def test_get_attribute_with_parent():
    el = GetConditions()
    el.get_element_attribute = MagicMock()
    el.get_element_attribute.return_value = 'sample return attribute'
    result = el.get_element_attribute(locator=MagicMock(), element='text')
    assert len(result) != 0


def test_get_element():
    el = GetConditions()
    el.get_element = MagicMock()
    el.get_element.return_value = 'sample return element'
    result = el.get_element(className=MagicMock())
    assert len(result) != 0

def test_text_should_be_enabled():
    el = GetConditions()
    el.text_should_be_enabled = MagicMock()
    el.text_should_be_enabled.return_value = True
    result = el.text_should_be_enabled(device=MagicMock(), className=MagicMock())
    assert result == True

def test_text_should_be_disabled():
    el = GetConditions()
    el.text_should_be_disabled = MagicMock()
    el.text_should_be_disabled.return_value = True
    result = el.text_should_be_disabled(device=MagicMock(), className=MagicMock())
    assert result == True

def test_element_should_contain_text():
    el = ExpectedConditions()
    el.element_should_contain_text = MagicMock()
    el.element_should_contain_text.return_value = True
    result = el.element_should_contain_text(device=MagicMock(), className=MagicMock(), text=MagicMock())
    assert result == True

def test_element_should_not_contain_text():
    el = ExpectedConditions()
    el.element_should_not_contain_text = MagicMock()
    el.element_should_not_contain_text.return_value = True
    result = el.element_should_not_contain_text(device=MagicMock(), className=MagicMock(), text=MagicMock())
    assert result == True

def test_check_element_visible():
    el = ExpectedConditions()
    el.check_element_visible = MagicMock()
    el.check_element_visible.return_value = True
    result = el.check_element_visible(device=MagicMock(), className=MagicMock(), text=MagicMock())
    assert result == True

def test_check_element_non_visible():
    el = ExpectedConditions()
    el.check_element_non_visible = MagicMock()
    el.check_element_non_visible.return_value = True
    result = el.check_element_non_visible(device=MagicMock(), className=MagicMock(), text=MagicMock())
    assert result == True

def test_get_element_with_device():
    el = GetConditions()
    el.get_element = MagicMock()
    el.get_element.return_value = 'sample return element'
    result = el.get_element(device=MagicMock(), className=MagicMock())
    assert len(result) != 0


def test_get_element_with_parent():
    el = GetConditions()
    el.get_element = MagicMock()
    el.get_element.return_value = 'sample return element'
    result = el.get_element(locator=MagicMock())
    assert len(result) != 0


def test_count_element():
    el = GlobalElement()
    el.count_elements = MagicMock()
    el.count_elements.return_value = 1
    result = el.count_elements(className=MagicMock())
    assert type(result) == int


def test_count_element_with_parent():
    el = GlobalElement()
    el.count_elements = MagicMock()
    el.count_elements.return_value = 1
    result = el.count_elements(locator=MagicMock())
    assert type(result) == int


def test_get_height():
    el = GetConditions()
    el.get_height = MagicMock()
    el.get_height.return_value = 1
    result = el.get_height()
    assert type(result) == int


def test_get_width():
    el = GetConditions()
    el.get_width = MagicMock()
    el.get_width.return_value = 1
    result = el.get_width()
    assert type(result) == int


def test_click_a_point():
    el = Click()
    el.click_a_point = MagicMock()
    el.click_a_point.return_value = True
    result = el.click_a_point(x=MagicMock(), y=MagicMock())
    assert result == True



import pytest
from alog import debug, info, error
from mock import MagicMock, call, patch
from mykonos.keywords.touch import Touch

def test_swipe():
    le = Touch()
    le.swipe = MagicMock()
    le.swipe.return_value = True
    result = le.swipe(sx = MagicMock(), sy = MagicMock(), ex = MagicMock(), ey = MagicMock(), steps = MagicMock())
    assert result == True

def test_swipe_device():
    le = Touch()
    le.swipe = MagicMock()
    le.swipe.return_value = True
    result = le.swipe(sx = MagicMock(), sy = MagicMock(), ex = MagicMock(), ey = MagicMock(), steps = MagicMock(), device = MagicMock())
    assert result == True

def test_swipe_direction():
    le = Touch()
    le.swipe_with_direction = MagicMock()
    le.swipe_with_direction.return_value = True
    result = le.swipe_with_direction(direction=MagicMock(), steps=100)

    assert result == True

def test_scroll():
    le = Touch()
    le.scroll = MagicMock()
    le.scroll.return_value = True
    assert le.scroll(steps=MagicMock()) == True
    assert le.scroll(steps=MagicMock(), textName=MagicMock()) == True
    assert le.scroll(steps=MagicMock(), textName=MagicMock(), action=MagicMock(), max_swipes=MagicMock()) == True

def test_pinch():
    le = Touch()
    le.pinch = MagicMock()
    le.pinch.return_value = True
    assert le.pinch(steps=100, textName=MagicMock(), action=MagicMock(), percent=100) == True

def test_fling():
    le = Touch()
    le.fling = MagicMock()
    le.fling.return_value = True
    assert le.fling(steps=MagicMock()) == True
    assert le.fling(steps=MagicMock(), action=MagicMock(), max_swipes=MagicMock()) == True



from mykonos.keywords.touch import Touch
from mykonos.keywords.element import GlobalElement, Click, GetConditions, ExpectedConditions
from mykonos.keywords.wait import Wait
from mykonos.keywords.decorators import Decorators
from mykonos.keywords.decorators import Parallel
from mykonos.keywords.key_event import KeyEvent
from mykonos.keywords.management_device import ManagementDevice


__all__ = [
    "Touch",
    "GlobalElement",
    "KeyEvent",
    "ManagementDevice",
    "Click",
    "GetConditions",
    "ExpectedConditions",
    "Wait",
    "Decorators"
    "Parallel"
]



import types
import multiprocessing
from mykonos.keywords.management_device import ManagementDevice


class Setup():

    def __init__(self):
        pass

    def __clean_dot_on_version(self, version_android=None):
        if '.' in version_android:
            version_android = version_android.split('.')
            return int(version_android.pop(0))
        else:
            return int(version_android)

    def __device_check(self, **settings):

        if 'device' in settings:
            get_device = settings['device']

            if isinstance(get_device, str):
                return next(ManagementDevice().get_android_version(device=get_device))
            else:
                versions = ManagementDevice().get_android_version(device=get_device)
                version_list = []
                for version in versions:
                    version_list.append(version)

                return version_list
        else:
            return next(ManagementDevice().get_android_version())

    def remove_dot_version(self, **settings):
        version_android = self.__device_check(**settings)
        list_version = []
        if isinstance(version_android, str):
            return self.__clean_dot_on_version(version_android)
        else:
            for version in version_android:
                list_version.append(self.__clean_dot_on_version(version))

            return list_version


class Decorators(object):

    def __init__(self):
        pass

    def android_version(func):

        def wrapper(self, *argument, **settings):
            management_device = ManagementDevice()
            setup = Setup()
            get_version_android = management_device.get_android_version()

            if len(list(get_version_android)) == 1 or 'device' in settings:

                version = setup.remove_dot_version(**settings)

                if 'text' in settings:
                    text = settings['text']
                    del settings['text']

                    if int(version) >= 9:
                        text_name = text.upper()
                        func(self, text=text_name, *argument, **settings)
                    else:
                        func(self, text=text, *argument, **settings)
                else:
                    func(self, *argument, **settings)


        return wrapper

class Parallel(object):

    def __init__(self):
        pass

    def device_check(func):

        def wrapper(self, *argument, **settings):

            if 'devices_parallel' in settings:
                devices_parallel = settings['devices_parallel']
                del settings['devices_parallel']

                list = []
                if isinstance(devices_parallel, str):
                    return func(self, device=devices_parallel, *argument, **settings)
                else:
                    list = [func(self, device=d, *argument, **settings) for d in devices_parallel]

                    return list

            else:
                return func(self, device=None, *argument, **settings)


        return wrapper



from robot.libraries.BuiltIn import BuiltIn
from mykonos.core.core import Core
from mykonos.keywords.management_device import ManagementDevice
from mykonos.keywords.decorators import Decorators, Parallel

class GlobalElement(Core):
    def __init__(self):
        self.get = GetConditions()
        self.device_mobile = self.device()
        self.management_device = ManagementDevice()

    @Parallel.device_check
    def open_notification(self, device=None, **settings):
        """Open notification a device.

        This keywords is used to open notification of device

        **Example:**
        || Open notification

        With Device/ Pararel :
        ||  @{emulator} =      |  192.168.1.1    | 192.168.1.2
        || Open notification   |device_parallel=@{emulator}


        """

        if device is not None:
            get_device = self.management_device.scan_current_device(device)
            return get_device().open.notification()
        else:
            return self.device_mobile.open.notification()

    @Parallel.device_check
    def open_quick_settings(self, device=None, **settings):
        """Open Quick Setting a device.

        This keywords is used to open setting of device

        **Example:**

        || Open Quick setting  |

        With Device/ Pararel :
        ||  @{emulator} =      |  192.168.1.1    | 192.168.1.2
        || Open Quick setting  |  device_parallel= @{emulator}

        """
        if device is not None:
            get_device = self.management_device.scan_current_device(device)
            return get_device().open.quick_settings()
        else:
            return self.device_mobile.open.quick_settings()

    @Parallel.device_check
    def clear_text(self, device=None, *argument, **settings):
        """Clear text on the text field base on locator.

        This keywords is used to clear text field.

        **Example:**

        ||Clear Text        |  className=sample class

        With Device/ Pararel :
        ||  @{emulator} =      |  192.168.1.1    | 192.168.1.2
        ||  Clear Text         | className=sample class     |device_parallel= @{emulator}
        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.clear_text()
        else:
            if 'device' is not None:
                get_device = self.management_device.scan_current_device(device)
                return get_device(*argument, **settings).clear_text()
            else:
                return self.device_mobile(*argument, **settings).clear_text()

    @Parallel.device_check
    def input_text(self, device=None, *argument, **settings):
        """Input text on the text field base on locator.

        This keywords is used to input text into text field.

        **Example:**

        || Input Text        |  className=sample class    input=text

        With Device/ Pararel :
        ||  @{emulator} =      |  192.168.1.1    | 192.168.1.2
        ||  Input Text         |  className=sample class | input=text  | device_parallel=@{emulator}

        """
        input = settings['input']
        del settings['input']

        if 'locator' in settings:
            locator = settings['locator']
            return locator.set_text(input)
        else:
            if device is not None:
                get_device = self.management_device.scan_current_device(device)
                return get_device(*argument, **settings).set_text(input)
            else:
                return self.device_mobile(*argument, **settings).set_text(input)

    @Parallel.device_check
    def count_elements(self, device=None, *argument, **settings):
        """Count total element from the page.

        This keywords is used to count total element on the device page.

        **Example:**

        || Count Elements          |  className=sample class

        With Device/ Pararel :
        ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
        ||  Count Element   | device_parallel=@{emulator}

        **Return:**

        Total of elements (int)

        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.count
        elif 'watcher' in settings:
            watcher = settings['watcher']
            del settings['watcher']

            return watcher.count
        else:
            if device is not None:
                get_device = self.management_device.scan_current_device(device)
                return get_device(*argument, **settings).count
            else:
                return self.device_mobile(*argument, **settings).count

    @Parallel.device_check
    def turn_on_screen(self, device=None, **settings):
        """Turn on Screen Device.

        **Example:**

        ||  Turn On Screen

        With Device/ Pararel :
        ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
        ||  Turn On Screen  | device_parallel=@{emulator}

        **Return:**

         True or False
        """
        if device is not None:
            get_device = self.management_device.scan_current_device(device)
            return get_device(**settings).screen.on()
        else:
            return self.device(**settings).screen.on()

    @Parallel.device_check
    def turn_off_screen(self, device=None, **settings):
        """Turn off Screen Device.

        **Example:**

        ||  Turn Off Screen

        With Device/ Pararel :
        ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
        ||  Turn Off Screen  | device_parallel=@{emulator}

        **Return:**

         True or False
        """
        if device is not None:
            get_device = self.management_device.scan_current_device(device)
            return get_device(**settings).screen.off()
        else:
            return self.device(**settings).screen.off()

    @Parallel.device_check
    def dump_xml(self, device=None, **settings):
        """Dump hierarchy of ui and will be saved as hierarchy.xml.

        **Example:**

        ||  Dump Xml        | file=sample.xml

        With Device /pararel :

        ||  @{emulator} =   |  192.168.1.1              | 192.168.1.2
        ||  Dump Xml        | file=sample.xml           | devices_pararel=@{emulator}

        **Return:**

        xml file of device
        """

        if 'file' in settings:
            file = settings['file']
            del settings['file']

        if device is not None:
            return self.management_device.scan_current_device(device).dump(file)
        else:
            return self.device().dump(file)

    @Parallel.device_check
    def capture_screen(self, file=None, device=None):
        """Capture screen of device testing.

        **Example:**

        ||  Capture Screen

        With file name:

        || Capture Screen        | file=sample

        With Device/ Pararel :
        ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
        ||   Capture Screen  | device_parallel=@{emulator}
        || Capture Screen    | file=sample  | device_parallel=@{emulator}

        **Return:**

        screen capture of device(*.png)
        """
        if file is not None:
            if device is not None:
                return self.device().screenshot(file+'.png')
            else:
                return self.management_device.scan_current_device(device).screenshot(file+'.png')
        else:
            index = 0
            index += 1
            filename = 'mykonos-screenshot-%d.png' % index
            if device is not None:
                return self.device().screenshot(filename)
            else:
                return self.management_device.scan_current_device(device).screenshot(filename)


class Click(Core):

    def __init__(self):
        self.device_mobile = self.device()
        self.management_device = ManagementDevice()

    @Parallel.device_check
    def click_element(self, device=None, *argument, **settings):
        """Click on UI base on locator.

        This keyword is used to click button or element of device.

        **Example:**

        ||  Click Element                    | className=sample class

        With Device/ Pararel :
        ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
        ||  Click Element                    | className=sample class  | device_parallel=@{emulator}
        """

        if 'locator' in settings:
            locator = settings['locator']
            return locator.click()

        else:
            if device is not None:
                get_devices = self.management_device.scan_current_device(device)
                return get_devices(*argument, **settings).click()

            elif 'watcher' in settings:
                watcher = settings['watcher']
                del settings['watcher']

                return watcher.click(*argument, **settings)
            else:
                return self.device_mobile(*argument, **settings).click()

    @Decorators.android_version
    def long_click_element(self, device=None, *argument, **settings):
        """Long click on UI base on locator.

        This keyword is used to long click button or element of device.


        **Example:**
        ||  Long Click Element                 | className=sample class

        With Device/ Pararel :
        ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
        ||  Long Click Element               | className=sample class  | device_parallel=@{emulator}

        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.long_click()
        else:
            if device is not None:
                get_devices = self.management_device.scan_current_device(device)
                return get_devices.long_click()
            else:
                return self.device_mobile(*argument, **settings).long_click()

    @Decorators.android_version
    def click_a_point(self, device=None, *argument, **settings):
        """Click into pointer target location.

         This keyword is used to click location  based on pointer X and Y.

         **Example:**

         ||  CLick A Point      |x=10   |y=20

         With Device/ Pararel :
         ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
         ||  CLick A Point   |x=10   |y=20    | device_parallel=@{emulator}

        """
        if 'x' in settings and 'y' in settings:
            x = settings['x']
            y = settings['y']
            del settings['x']
            del settings['y']
        else:
            raise ValueError('pointer x or y is refused')

        if device is not None:
            get_devices = self.management_device.scan_current_device(device)
            return get_devices.click(x, y)
        else:
            return self.device_mobile().click(x, y)


class GetConditions(Core):
    def __init__(self):
        self.device_mobile = self.device()
        self.management_device = ManagementDevice()

    @Parallel.device_check
    def get_info(self, device=None, value=None):
        """Get Info of Device.

        **Example:**

        ||  Get Info         |  value=displayRotation

         With Device/ Pararel :
         ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
         ||  Get Info        | value=displayRotation | device_parallel=@{emulator}

        **Return:**
        { u'displayRotation': 0,
          u'displaySizeDpY': 640,
          u'displaySizeDpX': 360,
          u'currentPackageName': u'com.android.launcher',
          u'productName': u'takju',
          u'displayWidth': 720,
          u'sdkInt': 18,
          u'displayHeight': 1184,
          u'naturalOrientation': True
        }
        """

        if value is None:
            result = self.device(device).info
        else:
            result = self.device(device).info[value]

        return result

    @Parallel.device_check
    def get_text(self, device=None, *argument, **settings):
        """Get text from element base on locator.

        **Example:**

        ||  Get Text         |  className=sample class

         With Device/ Pararel :
         ||  @{emulator} =   | 192.168.1.1    | 192.168.1.2
         ||  Get Text        | className=sample class | device_parallel=@{emulator}

        **Return:**
        String
        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.info['text']
        else:
            if device is not None:
                get_devices = self.management_device.scan_current_device(device)
                return get_devices(*argument, **settings).info['text']
            else:
                return self.device_mobile(*argument, **settings).info['text']

    @Parallel.device_check
    def get_element_attribute(self, device=None, *argument, **settings):
        """Get element attribute keyword of device.

        **List of Elements:**

         childCount, bounds, className, contentDescription,
         packageName, resourceName, text, visibleBounds,
         checkable, checked, clickable, enabled, focusable, disable,
         focused, longClickable, scrollable, selected

         **Example:**

         || Get Element Attribute    |  className=sample   | element=text

         **Return:**

         Attribute from element device
        """
        element = settings['element']
        del settings['element']

        if 'locator' in settings:
            locator = settings['locator']
            return locator.info[element]
        else:
            if device is not None:
                get_devices = self.management_device.scan_current_device(device)
                return get_devices(*argument, **settings).info[element]
            else:
                return self.device_mobile(*argument, **settings).info[element]

    @Parallel.device_check
    def get_element(self, device=None, *argument, **settings):
        """Get element info of device .
        This keyword is used to get element info of device.

         **Example:**

        || Get Element    |

        **Return:**

        {'currentPackageName': 'com.google.android.apps.nexuslauncher',
         'displayHeight': 1794,
         'displayRotation': 0,
         'displaySizeDpX': 411,
         'displaySizeDpY': 731,
         'displayWidth': 1080,
         'productName': 'sdk_google_phone_x86',
         'screenOn': True,
         'sdkInt': 25,
         'naturalOrientation': True}
        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.info
        else:
            if device is not None:
                get_devices = self.management_device.scan_current_device(device)
                return get_devices(*argument, **settings).info
            else:
                return self.device_mobile(*argument, **settings).info

    @Parallel.device_check
    def get_element_by_coordinate_x(self, device=None, *argument, **settings):
        """Get element by coordinate X.
        This keyword is used to get coordinate X of device.

         **Example:**

        || Get Element By Coordinate X  |  className=sample class

        **Return:**

        Coordinate x(int)
        """
        if device is not None:
            get_devices = self.management_device.scan_current_device(device)
            print(get_devices)
            # bound = self.get_element_attribute(element='bounds', device=get_devices, *argument, **settings)

        else:
            bound = self.get_element_attribute(element='bounds', *argument, **settings)
            bottom = bound['bottom']
            top = bound['top']
            elm_x = (top+bottom)+top
            return elm_x

    def get_element_by_coordinate_y(self, *argument, **settings):
        """Get element by coordinate Y.

        This keyword is used to get coordinate Y of device.

         **Example:**

        || Get Element By Coordinate Y  |  className=sample class

        **Return:**

        Coordinate y(int)
        """
        bound = self.get_element_attribute(element='bounds', *argument, **settings)
        display_height = self.get_height()
        height = display_height
        left = bound['left']
        right = bound['right']
        elm_y = height-(right+left)
        return elm_y

    def get_width(self):
        """Get width from display of device.

        This keyword is used to get widh of device,

        **Example:**

        || Get Width

        **Return:**

        Width of device(int)
        """
        get_device = self.device()
        return get_device.info['displayWidth']

    def get_height(self):
        """Get height from display of device.

        This keyword is used to get widh of device.

        **Example:**

        || Get Height

        **Return:**

        Width of device(int)
        """
        get_device = self.device()
        return get_device.info['displayHeight']

    def get_position(self, position=0, *argument, **settings):
        """Get Position of element.

        This keyword is used to get position of device element.

        **Example:**

        || Get Position       |  className=sample   | position=1

        **Return:**

        Width of device(int)
        """
        if 'locator' in settings:
            locator = settings['locator']
            del settings['locator']
            return locator[position]
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                return device(*argument, **settings)[position]
            else:
                return self.device_mobile(*argument, **settings)[position]


class ExpectedConditions(Core):
    def __init__(self):
        self.device_mobile = self.device()
        self.get_conditions = GetConditions()

    @Parallel.device_check
    def page_should_contain_element(self, device=None, *argument, **settings):
        """Page should contain element.
        The keyword is used to verify the page is contains locator element.

        **Example:**

        || Page Should Contain Element | className=sample class

        **Return:**

        True or False
        """
        # element = self.get_conditions.get_element(*argument, **settings)

        if 'locator' in settings:
            locator = settings['locator']
            if locator.exists:
                return True
            else:
                raise ValueError('locator not found')
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                return device(*argument, **settings).exists
            else:
                return self.device_mobile(*argument, **settings).exists

    @Parallel.device_check
    def page_should_contain_text(self, device=None, *argument, **settings):
        """Page should contain text.
        The keyword is used to verify the page is contains text.

        **Example:**

        || Page Should Contain Text | text=sample text

        **Return:**

        True or False
        """
        text = settings['text']

        if 'device' in settings:
            device = settings['device']
            del settings['device']
            return device(*argument, **settings).exists
        else:
            if device != None:
                get_devices = self.management_device.scan_current_device(device)
                return get_devices(*argument, **settings).exists

            elif 'watcher' in settings:
                watcher = settings['watcher']
                del settings['watcher']

                return watcher.exists(*argument, **settings)
            else:
                return self.device_mobile(*argument, **settings).exists

    @Parallel.device_check
    def __get_device_global(self, device=None, *argument, **settings):
        if 'locator' in settings:
            device = settings['locator']
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                device = device(*argument, **settings)
            else:
                device = self.device_mobile(*argument, **settings)

        return device

    @Parallel.device_check
    def page_should_not_contain_element(self, device=None, *argument, **settings):
        """Page should not contain element.

        The keyword is used to verify the page is not contains element.

        **Example:**

        || Page Should Not Contain Element | className=sample element

        **Return:**

        True or False
        """
        # element = self.get_conditions.get_element(*argument, **settings)

        if 'locator' in settings:
            locator = settings['locator']
            found = locator.exists
            if found is False:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                found = device(*argument, **settings).exists
                if found is True:
                    return False
                else:
                    return True
            else:
                found = self.device_mobile(*argument, **settings).exists
                if found is True:
                    return False
                else:
                    return True

    @Parallel.device_check
    def page_should_not_contain_text(self, device=None, *argument, **settings):
        """Page should not contain text.

        The keyword is used to verify the page is not contains text.

        **Example:**

        || Page Should Contain Text | text=sample text

        **Return:**

        True or False

        """
        text = self.get_conditions.get_text(*argument, **settings)

        if 'locator' in settings:
            locator = settings['locator']
            found = locator[text].exists
            if found is False:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                return device(*argument, **settings).exists
            else:
                return self.device_mobile(*argument, **settings).exists

    @Parallel.device_check
    def text_should_be_enabled(self, device=None, *argument, **settings):
        """Text should be enabled.

        The keyword is used to identify text enable.

        **Example:**

        || Text Should Be Enabled | text=sample text

        **Return:**

        True or False
        """
        element = self.get_conditions.get_element()
        enabled = element['enabled']
        if 'locator' in settings:
            locator = settings['locator']
            if locator.info['enabled'] is True:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            return self.device_mobile(*argument, **settings).enabled

    @Parallel.device_check
    def text_should_be_disabled(self, device=None, *argument, **settings):
        """Text should be disabled.

        The keyword is used to identify text disabled.

        **Example:**

        || Element Should Be Disabled | text=sample text

        **Return:**

        True or False
        """
        element = self.get_conditions.get_element()
        enabled = element['enabled']
        if 'locator' in settings:
            locator = settings['locator']
            if locator.info['enabled'] is False:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            return self.device_mobile(*argument, **settings).enabled

    @Parallel.device_check
    def element_should_contain_text(self, device=None, *argument, **settings):
        """Element should contain text.

        The keyword is used to identify text on element.

        **Example:**

        || Element Should Contain Text | className=class | text=sample text

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['text'] is not None:
                    return True
            except Exception:
                return False
        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['text'] is not None:
                    return True
            except Exception:
                return False

    @Parallel.device_check
    def element_should_not_contain_text(self, device=None, *argument, **settings):
        """Element should contain text.

        The keyword is used to identify text on element.

        **Example:**

        || Element Should Not Contain Text | className=class | text=sample text

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['text'] is None:
                    return True
                return False
            except Exception as error:
                return True

        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['text'] is None:
                    return True
                return False
            except Exception as error:
                return True

    @Parallel.device_check
    def check_element_visible(self, device=None, *argument, **settings):
        """Check element visible.

        The keyword is used to check element visible.

        **Example:**

        || Check Element Visible | className=sampleclassName

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['visibleBounds'] is None:
                    return False
                return True
            except Exception as error:
                return ("Exception Error: {0}".format(error))

        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['visibleBounds'] is None:
                    return False
                return True
            except Exception as error:
                return ("Exception Error: {0}".format(error))

    @Parallel.device_check
    def check_element_non_visible(self, device=None, *argument, **settings):
        """Check element non visible.

        The keyword is used to check element non visible.

        **Example:**

        || Check Element Non Visible | className=sampleclassName

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['visibleBounds'] is None:
                    return True
                return False
            except Exception as error:
                return ("Exception Error: {0}".format(error))

        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['visibleBounds'] is None:
                    return True
                return False
            except Exception as error:
                return ("Exception Error: {0}".format(error))



import traceback
from alog import debug, info, error
from mykonos.core.core import Core


class LocatorElement(Core):

    def __init__(self):
        self.device_mobile = self.device()

    def get_locator(self, *argument, **settings):
        """Access locator from device.

        **selector support:**

        * text, textContains, textMatches, textStartsWith
        * className, classNameMatches
        * description,descriptionContains,descriptionMatches,descriptionStartsWith
        * checkable, checked, clickable, longClickable
        * scrollable, enabled,focusable, focused, selected
        * packageName, packageNameMatches
        * resourceId, resourceIdMatches
        * index, instance


        **Example:**

        || ${locator}= Get Locator           | text=sample text

        """
        if 'device' in settings:
            device = settings['device']
            del settings['device']
            return device(*argument, **settings)
        else:
            return self.device_mobile(*argument, **settings)

    def get_child(self, parent, *argument, **settings):
        """Access child locator from device.

        **Example:**

        || ${locator}=  Get Locator        | text=sample text

        || ${child}=   Get Child           | parent=${locator}  text=sample text
        """
        return parent.child(*argument, **settings)

    def get_sibling(self, parent, *argument, **settings):
        """Access sibling locator from device.

        **Example:**

        || ${locator}=  Get Locator        | text=sample text

        || ${sibling}=   Get Sibiling      | parent=${locator}  text=sample text
        """
        return parent.sibling(*argument, **settings)

    def left_position(self, parent, *argument, **settings):
        """Access left position from device.

        **Example:**

        || ${locator}=  Get Locator        | text=sample text

        || ${left}=     Left Position      | parent=${locator}  text=sample text
        """
        return parent.left(*argument, **settings)

    def right_position(self, parent, *argument, **settings):
        """Access left position from device.

        **Example:**

        || ${locator}=  Get Locator        | text=sample text

        || ${right}=    Right Position     | parent=${locator}  text=sample text
        """
        return parent.right(*argument, **settings)

    def up_position(self, parent, *argument, **settings):
        """Access left position from device.

        **Example:**

        || ${locator}=  Get Locator        | text=sample text

        || ${up}=    Up Position           | parent=${locator}  text=sample text
        """
        return parent.up(*argument, **settings)

    def down_position(self, parent, *argument, **settings):
        """Access left position from device.

        **Example:**

        || ${locator}=  Get Locator        | text=sample text

        || ${down}=    Down Position       | parent=${locator}  text=sample text
        """
        return parent.down(*argument, **settings)

    def get_locator_by_index(self, *argument, **settings):
        """Get Element locator by index on device.

        **Example:**

        || Get Locator By Index           | text=sample_text   | index=1

        || ${locator}=  Get Locator        | text=sample text

        || Get Locator By Index            | locator=${locator} | index=1
        """
        index = int(settings['index'])
        del settings['index']

        if 'locator' in settings:
            locator = settings['locator']
            del settings['locator']

            return locator[index]
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                return device(*argument, **settings)[index]
            else:
                return self.device_mobile(*argument, **settings)[index]

    def handlers(self, action, function):
        """Call customized function on device.

        **Example:**

        || Handlers                  | action=on   | function=sample_function
        """
        if 'on' in action:
            return self.device_mobile.handlers.on(function)
        elif 'off' in action:
            return self.device_mobile.handlers.off(function)


class WatcherElement(Core):
    """Class is used to perform some actions when selector cannot be found."""

    def __init__(self):
        self.device_mobile = self.device()

    def __watcher_register(self, **settings):
        name = settings['name']
        del settings['name']
        return self.device_mobile.watcher(name)

    def watcher(self, **settings):
        """Watcher is registered when a selector cannot be find.

        name=name of watcher

        WHEN, className=sample_class

        WHEN, packageName=sample_package

        **Example:**

        || ${sample_watcher}=name=sample_watcher  | className=sample_class

        || Click Element   | watcher=${sample_watcher}      | text=sample_text
        """
        name_watcher = settings['name']
        del settings['name']
        self.__watcher = self.__watcher_register(name=name_watcher)
        return self.__watcher.when(**settings)

    def watcher_action(self, action, **settings):
        """Watcher Action is used to running the action on the watcher.

        run=Force to run all watchers

        remove=Remvoe watchers

        reset=Reset all triggered watchers

        list=List all watchers

        triggered=Check if there is any watcher triggered


        **Example:**

        || Watcher Action  | action=run

        || Watcher Action  | action=remove

        || Watcher Action  | action=remove | name=sample_watcher

        || Watcher Action  | action=reset

        || Watcher Action  | action=list

        || Watcher Action  | action=triggered
        """
        if 'run' in action:
            return self.device_mobile.watchers.run()
        elif 'remove' in action:
            if 'name' in settings:
                name = settings['name']
                del settings['name']
                return self.device_mobile.watchers.remove(name)
            else:
                return self.device_mobile.watchers.remove()
        elif 'list' in action:
            return self.device_mobile.watchers
        elif 'reset' in action:
            return self.device_mobile.watchers.reset()
        elif 'triggered' in action:
            return self.device_mobile.watchers.triggered



import os
import subprocess
from mykonos.core.core import Core
from mykonos.keywords.management_device_utils import DeviceUtils

class ManagementDevice(Core):
    adb_s = 'adb -s '
    adb_start = ' shell am start -W '
    adb_stop = ' shell am force-stop '
    adb_grep_pid = 'adb shell pgrep '
    adb_kill = 'adb shell pkill '
    adb_pm_clear = 'shell pm clear'
    adb_key_event = 'adb shell input keyevent '
    adb_pull = 'adb pull '
    adb_push = 'adb push '
    adb_activity = 'adb shell dumpsys activity | grep '
    adb_disable = 'adb shell pm disable'
    adb_check_version = 'adb shell getprop ro.build.version.release'
    adb_devices = 'adb devices | grep -v devices | grep device | cut -f 1'

    def __init__(self):
        """Devine all global variable."""
        self.index = 0

    def get_devices(self):
        list = []
        cmd = 'adb devices'
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        byte_decode = out.decode()
        # Get Index
        start = [i+1 for i in range(len(byte_decode)) if byte_decode.find('\n', i) == i]
        end = [i for i in range(len(byte_decode)) if byte_decode.find('\t', i) == i]

        if len(start) < len(end):
            total = len(start)
        else:
            total = len(end)
        for i in range(0, total):
            list.append(byte_decode[start[i]:end[i]])
        return list

    def scan_current_device(self,  *args, **settings):
        """Scan current device on the workstation, and consume to open application.
        **Example:**
        ||  Scan Current Device                        |  emulator-554
        """
        return self.device(*args)

    def open_app(self, device, package):
        """Open Application on device.
        This keyword is used to open new applications.
        **Example:**
        ||  Open Application      |  device=emulator-554   | package=sample_apk
        """
        try:
            os.system(self.adb_s + device + self.adb_start + package + '')
            return self.device(device)
        except ValueError:
            raise ValueError('open device is failed')

    def _substring_package(self, package):
        return package.split('/')[0]

    def quit_app(self, device, package):
        """Quit application on device.
        This keyword is used to close application without kill a session.
        **Example:**
        ||  Quit App      |  device=emulator-554   | package=sample_apk
        """
        package = self._substring_package(package)
        cl = os.system(self.adb_kill + package)
        return cl

    def close_all_app(self, device):
        """Close all tasks on device, and kill all application sessions.
        **Example:**
        || Close All App      |  device=emulator-554   |
        """
        try:
            os.system('adb -s '+device+' shell am kill-all')
            return self.device(device)
        except ValueError:
            raise ValueError('device can not be opened')

    def __append_current_package(self, cmd):
        top_activity = self.__shell_pipe(cmd)
        start_pid = str(top_activity).find("'")
        end_pid = str(top_activity).find(":")
        end_package_name = str(top_activity).find("/")
        pid = str(top_activity)[start_pid:end_pid]
        package = str(top_activity)[end_pid+1:end_package_name]
        return package


    def __get_current_package(self, **settings):
        get_device = self.get_devices()
        if 'device' in settings:
            device = settings['device']
            out = self.__shell_pipe(cmd="adb -s %s shell dumpsys activity | grep top-activity | awk '{ print $8 }'")
            self.__append_current_package(out)
        else:
            if len(get_device) == 1:
                out = self.__shell_pipe(cmd="adb shell dumpsys activity | grep top-activity | awk '{ print $8 }'")
                self.__append_current_package(out)
            else:
                for i in get_device:
                    out = self.__shell_pipe(cmd="adb -s %s shell dumpsys activity | grep top-activity | awk '{ print $8 }'")
                    self.__append_current_package(out)

    def reset_app(self, device, package):
        """Reset Application on Device.
        This keyword is used to reset the current application while sesion is keep alive.
        **Example:**
        || Reset Application   |  emulator=emulator-554 | package=sample_apk
        """
        try:
            reset = self.__shell_pipe(cmd='adb -s %s shell pm clear %s' % (device, package))
            # reset = os.system('adb -s'+device+' shell pm clear '+package)
            return reset
        except ValueError:
            raise ValueError('reset apps is failed')

    def hide_keyboard(self):
        """Hide Keyword on Device.
        This keyword is used to hide keyboard device.
        **Example:**
        || Hide keyboard      |
        """
        rs = os.system(self.adb_key_event+'111')
        return rs

    def pull(self, **settings):
        """Pull file from Device.
        This Keyword is used to retrieves file from device.
        **Example:**
        || Pull         | local=sample_path  | remote=sample_location
        || Pull         | local=sample_path  |
        """
        local = settings['local']
        if 'remote' in settings:
            remote = settings['remote']
            rs = os.system(self.adb_pull + local + remote)
        else:
            rs = os.system(self.adb_pull + local)
        if rs != 0:
            return False
        else:
            return True

    def push(self, **settings):
        """Push file into Device.
        This keyword is used to put file in spesific path of device.
        **Example:**
        || Push         | local=sample_path  | remote=sample_location
        """
        local = settings['local']
        remote = settings['remote']
        rs = os.system(self.adb_push + local + ' ' + remote)
        if rs != 0:
            return False
        else:
            return True

    def __shell_pipe(self, cmd):
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
        return out

    def __return_dumpsy_package(self, cmd):
        out = self.__shell_pipe(cmd)
        start = str(out).find('{')
        end = str(out).find('/')
        slice_1 = str(out)[start:end]
        result = str(slice_1)[str(slice_1).find(':')+1:]
        return result

    def __get_old_package(self):
        return self.__return_dumpsy_package(self.adb_activity + 'mPreviousProcess')

    def __adb_clear_cache(self):
        clear_cache = self.__shell_pipe('adb shell pm trim-caches')
        return clear_cache

    def switch_application(self, device, new_app):
        """Switch application the devices.
        This keywords return previous active application
        and it can be used in the next application.
        **Example:**
        || Switch Application      | device=sample_device  | new_app=sample_app
        """
        old = self.__get_old_package()
        new = new_app.info['currentPackageName']
        current = self.device().info['currentPackageName']
        result = self.open_app(device, old)
        return result

    def close_app(self, device, package):
        """Close Application the device.
        This keywords is used to close the current application and kill session on device.
        **Example:**
        || Close App        | devices=${emulator} | package=Package Activity
        """
        try:
            closed = self.__shell_pipe(cmd='adb -s %s shell am force-stop %s' % (device, package))
            # closed = os.system('adb -s '+device+' shell am force-stop '+package)
            return closed
        except ValueError:
            raise ValueError('device not found')

    def get_android_version(self, **settings):
        get_device = self.get_devices()
        if 'device' in settings:
            device = settings['device']
            out = self.__shell_pipe(cmd='adb -s %s shell getprop ro.build.version.release'%device)
            yield(out.decode().replace('\n', ''))
        else:
            if len(get_device) == 1:
                out = self.__shell_pipe(cmd=self.adb_check_version)
                yield(out.decode().replace('\n', ''))
            else:
                for i in get_device:
                    out = self.__shell_pipe(cmd='adb -s %s shell getprop ro.build.version.release'%i)
                    yield(out.decode().replace('\n', ''))



import pytest
from mock import MagicMock, call, patch
from pytest_mock import mocker
from mykonos.keywords.management_device import ManagementDevice

def test_scan_device():
    device = ManagementDevice()
    device.scan_device = MagicMock()
    device.scan_device.return_value = True
    result = device.scan_device()
    assert result == True

def test_open_app():
    device = ManagementDevice()
    device.open_app = MagicMock()
    device = MagicMock()
    device.open_app.return_value = True
    result = device.open_app(device=MagicMock(), app_package=MagicMock())
    assert result == True

def test_quite_app():
    device = ManagementDevice()
    device.quit_app = MagicMock()
    device.quit_app.return_value = True
    result = device.quit_app(device=MagicMock(), app_package=MagicMock())
    assert result == True

def test_reset_app():
    device = ManagementDevice()
    device.reset_app = MagicMock()
    device.reset_app.return_value = True
    result = device.reset_app(device=MagicMock(), app_package=MagicMock())
    assert result == True

def test_pull_file():
    device = ManagementDevice()
    device.pull = MagicMock()
    device.pull.return_value = True
    result = device.pull(local=MagicMock(), remote=MagicMock())
    assert result == True

def test_get_android_version():
    device = ManagementDevice()
    device.get_android_version = MagicMock()
    device.get_android_version.return_value = '9'
    result = device.get_android_version()
    assert result == '9'



import os
from mykonos.locator import LocatorElement, WatcherElement
from mykonos.keywords import ManagementDevice, GlobalElement, KeyEvent, Touch, Click, GetConditions, ExpectedConditions, Wait, Decorators
from mykonos.keywords import Parallel

class mykonos(
    ManagementDevice,
    LocatorElement,
    GlobalElement,
    KeyEvent,
    Touch,
    WatcherElement,
    Click,
    GetConditions,
    ExpectedConditions,
    Wait,
    Decorators,
    Parallel
):

    def __init__(self):
        for base in mykonos.__bases__:
            base.__init__(self)



import pytest
from pytest_mock import mocker
from mock import MagicMock, call, patch
from mykonos.keywords.key_event import KeyEvent
from mykonos.keywords.management_device import ManagementDevice


def test_press_menu():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("menu")
    print(result)
    assert result == True

def test_press_back():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("back")
    print(result)
    assert result == True

def test_press_left():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("left")
    print(result)
    assert result == True

def test_press_right():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("right")
    print(result)
    assert result == True

def test_press_up():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("up")
    print(result)
    assert result == True

def test_press_down():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("down")
    print(result)
    assert result == True


def test_press_center():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("center")
    print(result)
    assert result == True

def test_press_search():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("search")
    print(result)
    assert result == True

def test_press_home():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("home")
    print(result)

def test_press_volume_up():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("volume_up")
    print(result)
    assert result == True

def test_press_volume_down():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("volume_down")
    print(result)
    assert result == True

def test_press_camera():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("camera")
    print(result)
    assert result == True

def test_press():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode(0x07, 0x02)
    print(result)
    assert result == True

def test_press_with_device():
    cls = KeyEvent()
    press_keycode = MagicMock()
    cls.press_keycode = MagicMock()
    cls.press_keycode.return_value = True
    result = cls.press_keycode("volume_down", device=MagicMock())
